/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "ObjectViewTasks.h"

#include "LoadDocumentTask.h"

#include <core_api/Log.h>
#include <core_api/DocumentModel.h>
#include <core_api/AppContext.h>

namespace GB2 {

/* TRANSLATOR GB2::ObjectViewTask */

ObjectViewTask::ObjectViewTask(GObjectView* _view, const QString& stateName, const QVariantMap& s) 
: Task("", TaskFlags_NR_DWF), taskType(Type_Update), stateData(s), view(_view), stateIsIllegal(false)
{
    assert(view!=NULL);
    const QString& vName = view->getName();
    setTaskName(tr("Update '%1' to '%2' state").arg(vName).arg(stateName));
    setVerboseLogMode(true);
}

ObjectViewTask::ObjectViewTask(GObjectViewFactoryId fid, const QString& vName, const QVariantMap& s) 
: Task("", TaskFlags_NR_DWF), taskType(Type_Open), stateData(s), view(NULL), viewName(vName), stateIsIllegal(false)
{
    if (vName.isEmpty()) {
        QString factoryName = AppContext::getObjectViewFactoryRegistry()->getFactoryById(fid)->getName();
        setTaskName(tr("Open new '%1'").arg(factoryName));
    } else {
        setTaskName(tr("Open '%1'").arg(vName));
    }
    
    setVerboseLogMode(true);
}


void ObjectViewTask::prepare() {
    QSet<Document*> processed;
	foreach(QPointer<Document> pd, documentsToLoad) {
		if (!pd.isNull() && !processed.contains(pd)){
			addSubTask(new LoadUnloadedDocumentTask(pd));
            processed.insert(pd);
		}
	}
}

Task::ReportResult ObjectViewTask::report() {
	foreach(QPointer<Document> pd, documentsToLoad) {
		if (pd.isNull()){
			continue;
		}
		if (!pd->isLoaded()) {
			documentsFailedToLoad.append(pd);
			continue;
		} 
		onDocumentLoaded(pd);
	}

	if (taskType == Type_Open) {
		open();
    } else {
        assert(taskType == Type_Update);
		update();
	} 
	
	if (stateIsIllegal) {
		//todo: ask to remove illegal state
	}

	return ReportResult_Finished;		
}

} // namespace
