/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _GB2_WORKFLOW_MNGR_H_
#define _GB2_WORKFLOW_MNGR_H_

#include <workflow/WorkflowModel.h>
#include <workflow/WorkflowTransport.h>

#include <core_api/Task.h>

#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtCore/QVariant>
#include <assert.h>

namespace GB2 {

namespace Workflow {

class WCOREAPI_EXPORT Worker {
public:
    virtual void init() = 0;
    virtual bool isReady() = 0;
    virtual Task* tick() = 0;
    virtual bool isDone() = 0;
    virtual void cleanup() = 0;
    virtual ~Worker() {}
};

class WCOREAPI_EXPORT Scheduler : public Worker {};


class WCOREAPI_EXPORT DomainFactory : public IdRegistry<DomainFactory>, public Descriptor {
public:
    DomainFactory(const Descriptor& d) : Descriptor(d) {}
    DomainFactory(const QString& id) : Descriptor(id) {}
    virtual Worker* createWorker(Actor*) = 0;
    virtual CommunicationChannel* createConnection(Link*) {return NULL;}
    virtual Configuration* getDefaultSettings() {return NULL;}
    virtual Scheduler* createScheduler(Schema*, Configuration*) {return NULL;}
    virtual void destroy(Scheduler*, Schema*) {}
    virtual ~DomainFactory(){}
};

class WCOREAPI_EXPORT DomainRegistry : public IdRegistry<DomainFactory> {};

class WCOREAPI_EXPORT Manager {
public:
    //TBD
    void start(Schema*);
    void validate();
};

}//Workflow namespace
}//GB2 namespace


#endif
